# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2022 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:	modules/NovellLum.ycp
# Package:	Configuration of novell-lum
# Summary:	NovellLum settings, input and output functions
# Authors:	Stephen T. Bunch <sbunch@novell.com>
#
# $Id: NovellLum.ycp,v 1.1.2.47 2005/10/04 21:35:13 rhaslam Exp $
#
# Representation of the configuration of novell-lum.
# Input and output routines.
require "yast"

module Yast
  class NovellLumClass < Module
    def main
      Yast.import "UI"
      textdomain "novell-lum"

      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "Popup"
      Yast.import "String"
      Yast.import "Edirectory"
      Yast.import "EdirUtils"
      Yast.import "NovellUtils"
      Yast.import "Service"
      Yast.import "Misc"
      Yast.import "NovellLDAP"
      Yast.import "ValidationHelpers"
      Yast.import "NovellPatterns"
      Yast.import "NovellAddonInstall"
      Yast.import "Directory"
      Yast.import "Wizard"

      # Settings Map
      @Settings = {}
      @oldSettings = {}
      @old_EdirSettings = {}
      @old_XadSettings = {}
      @auto_clone = false
      @xad_packages_installed = false
      @edir_service_configured = false

      @xadMode = false
      @xadTestMode = false

      @schemaLogPath = "/var/opt/novell/eDirectory/log/oes_schema.log"
      @oesContext = "ou=OESSystemObjects"
      @lum_proxy = false
      @userChangedData = false
      @userChangedAdminGroup = false
      @doRecalculate = true

      # Data was modified?
      @modified = false

      @proposal_valid = false

      @upgrade_from = ""

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false
      @globals_configured = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    def PollAbort
      UI.PollInput == :abort
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # Read a setting (option) from a specified config file
    # Removes any extra quotes around values and avoids passing a nil value
    #
    # @param [Yast::Path] configpath scr path to the config file
    # @param [String] optname string key of the setting(option) we are reading
    # @return [String] value of the specfied setting
    def ReadOption(configpath, optname)
      optvalue = Convert.to_string(
        SCR.Read(Ops.add(configpath, Builtins.topath(optname)))
      )

      if optvalue == nil
        optvalue = "" # avoid nil values
      elsif Builtins.regexpmatch(optvalue, "'.*'")
        # remove the single quotes
        tmpoptvalue = Builtins.regexpsub(optvalue, "'(.*)'", "\\1")
        optvalue = tmpoptvalue if optvalue != nil || optvalue != ""
      elsif Builtins.regexpmatch(optvalue, "\".*\"")
        # remove the double quotes
        tmpoptvalue = Builtins.regexpsub(optvalue, "\"(.*)\"", "\\1")
        optvalue = tmpoptvalue if optvalue != nil || optvalue != ""
      end

      optvalue
    end

    # Change UMASK in /etc/login.defs file from 022 to 077
    # @return true on success
    def ChangeUmask
      retVal = true

      # Modify the /etc/login.defs file to change UMASK 022 to UMASK 077
      #Command executed from command line: sed -i 's/\([\t ]*UMASK[\t ]*\).*/\1 077/' /etc/login.defs;
      execute_cmd = "sed -i 's/\\([\\t ]*UMASK[\\t ]*\\).*/\\1 077/' /etc/login.defs"
      Builtins.y2milestone("NovellLum:Executing: %1", execute_cmd)
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      retVal = false if rval != 0

      retVal
    end

    def GetDomainType
      if (Mode.update || Mode.autoupgrade) && @xadMode # both sp1 and sp2 had dsfw
        Builtins.y2milestone(
          "lum.GetDomainType:xad upgmodeoes2 returning %1",
          Ops.get_string(@old_XadSettings, "XAD_DOMAIN_TYPE", "")
        )
        return Ops.get_string(@old_XadSettings, "XAD_DOMAIN_TYPE", "")
      end

      domainType = ""
      if Mode.installation && Stage.normal && @edir_service_configured == true
        domainType = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_TYPE"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "NovellLum.GetDomainType (post install):domain type from xad conf file:%1",
          domainType
        )
      else
        domainType = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_DOMAIN_TYPE",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.GetDomainType:domain type from eDir map:%1",
          domainType
        )
        if domainType == nil || domainType == ""
          domainType = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_TYPE"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "NovellLum.GetDomainType:domain type from xad conf file:%1",
            domainType
          )
        end
      end
      if domainType == nil || domainType == ""
        tmpMap = Edirectory.ProposeXadSettings # worst case, propose the system domain type.
        domainType = Ops.get_string(tmpMap, "XAD_DOMAIN_TYPE", "")
        Builtins.y2milestone(
          "NovellLum.GetDomainType:domain type from propose settings:%1",
          domainType
        )
      end
      domainType
    end

    def GetDomainName
      if (Mode.update || Mode.autoupgrade) && @xadMode
        Builtins.y2milestone(
          "lum.GetDomainName:xad upgmodeoes2 returning %1",
          Ops.get_string(@old_XadSettings, "XAD_DOMAIN_NAME", "")
        )
        return Ops.get_string(@old_XadSettings, "XAD_DOMAIN_NAME", "")
      end

      domainName = ""
      if Mode.installation && Stage.normal && @edir_service_configured == true
        domainName = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_NAME"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "NovellLum.GetDomainName (post install): domain name from xad conf file:%1",
          domainName
        )
      else
        domainName = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_DOMAIN_NAME",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.GetDomainName:domain name from eDir map:%1",
          domainName
        )
        if domainName == nil || domainName == ""
          domainName = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_NAME"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "NovellLum.GetDomainName: domain name from xad conf file:%1",
            domainName
          )
        end
      end
      if domainName == nil || domainName == ""
        tmpMap = Edirectory.ProposeXadSettings # worst case, propose the system domain name.
        domainName = Ops.get_string(tmpMap, "XAD_DOMAIN_NAME", "")
        Builtins.y2milestone(
          "NovellLum.GetDomainName:domain name from system domain name:%1",
          domainName
        )
      end
      domainName
    end

    def GetXadDomainName
      domainName = GetDomainName()
      split = Builtins.splitstring(domainName, ".")
      xadDomainName = Ops.add("dc=", Builtins.mergestring(split, ",dc="))
      Builtins.y2milestone(
        "NovellLum.GetXadDomainName:xadDomainName: %1",
        xadDomainName
      )
      xadDomainName
    end


    def GetNameMap
      if (Mode.update || Mode.autoupgrade) && @xadMode
        Builtins.y2milestone(
          "lum.GetNameMap:xad upgmodeoes2 returning %1",
          Ops.get_string(@old_XadSettings, "XAD_CONVERT_EXISTING_CONTAINER", "")
        )
        return Ops.get_string(
          @old_XadSettings,
          "XAD_CONVERT_EXISTING_CONTAINER",
          ""
        )
      end

      nameMap = ""
      if Mode.installation && Stage.normal && @edir_service_configured == true
        nameMap = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_CONVERT_EXISTING_CONTAINER"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "NovellLum.GetNameMap (post install):are we name mapped from xad conf file:%1",
          nameMap
        )
      else
        nameMap = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_CONVERT_EXISTING_CONTAINER",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.GetXadObjects:are we name mapped from eDir map:%1",
          nameMap
        )
        if nameMap == nil || nameMap == ""
          nameMap = Convert.to_string(
            SCR.Read(
              path(".sysconfig.novell.xad.XAD_CONVERT_EXISTING_CONTAINER")
            )
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "NovellLum.GetNameMap:are we name mapped from xad conf file:%1",
            nameMap
          )
        end
      end
      nameMap = "no" if nameMap == nil || nameMap == ""

      nameMap
    end

    def GetForestRoot
      if (Mode.update || Mode.autoupgrade) && @xadMode
        Builtins.y2milestone(
          "lum.GetForestRoot:xad upgmodeoes2 returning %1",
          Ops.get_string(@old_XadSettings, "XAD_FOREST_ROOT", "")
        )
        return Ops.get_string(@old_XadSettings, "XAD_FOREST_ROOT", "")
      end

      forestRoot = ""
      if Mode.installation && Stage.normal && @edir_service_configured == true
        forestRoot = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_FOREST_ROOT"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "NovellLum.GetForestRoot (post install):forest root from xad conf file:%1",
          forestRoot
        )
      else
        forestRoot = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_FOREST_ROOT",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.GetForestRoot:forest root from eDir map:%1",
          forestRoot
        )
        if forestRoot == nil || forestRoot == ""
          forestRoot = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_FOREST_ROOT"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "NovellLum.GetForestRoot:forest root from xad conf file:%1",
            forestRoot
          )
        end
      end
      forestRoot
    end

    def GetNameMapContainer
      if (Mode.update || Mode.autoupgrade) && @xadMode
        Builtins.y2milestone(
          "lum.GetNameMapContainer:xad upgmodeoes2 returning %1",
          Ops.get_string(@old_XadSettings, "XAD_EXISTING_CONTAINER", "")
        )
        return NovellUtils.Nds2Ldap(
          Ops.get_string(@old_XadSettings, "XAD_EXISTING_CONTAINER", "")
        )
      end

      nameMapContainer = ""
      if Mode.installation && Stage.normal && @edir_service_configured == true
        nameMapContainer = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_EXISTING_CONTAINER"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "NovellLum.GetNameMapContainer (post install):name mapped container from xad conf file:%1",
          nameMapContainer
        )
      else
        nameMapContainer = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_EXISTING_CONTAINER",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.GetNameMapContainer:name mapped container from eDir map:%1",
          nameMapContainer
        )
        if nameMapContainer == nil || nameMapContainer == ""
          nameMapContainer = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_EXISTING_CONTAINER"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "NovellLum.GetNameMapContainer:name mapped container from xad conf file:%1",
            nameMapContainer
          )
        end
      end
      if nameMapContainer == nil || nameMapContainer == ""
        nameMapContainer = ""
      else
        nameMapContainer = NovellUtils.Nds2Ldap(nameMapContainer)
      end

      nameMapContainer
    end

    def GetTreeType
      if (Mode.update || Mode.autoupgrade) && @xadMode
        Builtins.y2milestone(
          "lum.GetTreeType:xad upg2oes2 returning %1",
          Ops.get_string(@old_EdirSettings, "CONFIG_EDIR_TREE_TYPE", "")
        )
        return Ops.get_string(@old_EdirSettings, "CONFIG_EDIR_TREE_TYPE", "")
      end
      treeType = ""
      if Mode.installation && Stage.normal && @edir_service_configured == true
        treeType = Convert.to_string(
          SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
        ) # next try reading the edir conf file
        Builtins.y2milestone(
          "NovellLum.GetTreeType( post install ):tree type from edir conf file:%1",
          treeType
        )
      else
        treeType = Ops.get_string(
          Edirectory.EdirSettings,
          "CONFIG_EDIR_TREE_TYPE",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.GetTreeType:tree type from eDir map:%1",
          treeType
        )
        if treeType == nil || treeType == ""
          treeType = Convert.to_string(
            SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
          ) # next try reading the edir conf file
          Builtins.y2milestone(
            "NovellLum.GetTreeType:tree type from edir conf file:%1",
            treeType
          )
        end
      end
      if treeType == nil || treeType == ""
        treeType = Edirectory.ProposeTreeType # worst case, propose the tree type.
        Builtins.y2milestone(
          "NovellLum.GetTreeType:tree type from propose tree:%1",
          treeType
        )
      end
      treeType
    end

    def GetServerContext
      if (Mode.update || Mode.autoupgrade) && @xadMode
        Builtins.y2milestone(
          "lum.GetServerContext:xad upg2oes2 returning %1",
          Ops.get_string(@old_EdirSettings, "CONFIG_EDIR_SERVER_CONTEXT", "")
        )
        return Ops.get_string(
          @old_EdirSettings,
          "CONFIG_EDIR_SERVER_CONTEXT",
          ""
        )
      end

      serverContext = ""
      if Mode.installation && Stage.normal && @edir_service_configured == true
        serverContext = Convert.to_string(
          SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"))
        ) # next try reading the edir conf file
        Builtins.y2milestone(
          "NovellLum.GetServerContext (post install mode):server context from edir conf file:%1",
          serverContext
        )
      else
        serverContext = Ops.get_string(
          Edirectory.EdirSettings,
          "CONFIG_EDIR_SERVER_CONTEXT",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.GetServerContext:server context from eDir map:%1",
          serverContext
        )
        if serverContext == nil || serverContext == ""
          serverContext = Convert.to_string(
            SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"))
          ) # next try reading the edir conf file
          Builtins.y2milestone(
            "NovellLum.GetServerContext:server context from edir conf file:%1",
            serverContext
          )
        end
      end
      NovellUtils.Nds2Ldap(serverContext)
    end

    def GetAdminUser
      adminUser = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        ""
      )
      Builtins.y2milestone(
        "NovellLum.GetAdminUser:admin user from ldap map:%1",
        adminUser
      )
      if adminUser == nil || adminUser == ""
        cfgPath = "/etc/sysconfig/novell/oes-ldap"
        cfgfile_path = Ops.add(
          path(".etc.sysconfig.novell") + path(".value"),
          Builtins.topath(cfgPath)
        )
        Builtins.y2milestone(
          "NovellLum.getAdminUser:cfgfile_path %1",
          cfgfile_path
        )

        # Read in the global ldap config file settings
        adminUser = ReadOption(cfgfile_path, "CONFIG_LDAP_ADMIN_CONTEXT")
        Builtins.y2milestone(
          "NovellLum.GetAdminUser:admin user from ldap conf file:%1",
          adminUser
        )
      end
      NovellUtils.Nds2Ldap(adminUser)
    end

    def GetXadAdminUser
      adminUser = Ops.get_string(
        NovellLDAP.Settings,
        "XAD_TREE_ADMIN_CONTEXT",
        ""
      )
      Builtins.y2milestone(
        "NovellLum.GetXadAdminUser:xad admin user from ldap map:%1",
        adminUser
      )
      if adminUser == nil || adminUser == ""
        cfgPath = "/etc/sysconfig/novell/oes-ldap"
        cfgfile_path = Ops.add(
          path(".etc.sysconfig.novell") + path(".value"),
          Builtins.topath(cfgPath)
        )
        Builtins.y2milestone(
          "NovellLum.getXadAdminUser:cfgfile_path %1",
          cfgfile_path
        )

        # Read in the global ldap config file settings
        adminUser = ReadOption(cfgfile_path, "XAD_TREE_ADMIN_CONTEXT")
        Builtins.y2milestone(
          "NovellLum.GetXadAdminUser:xad admin user from ldap conf file:%1",
          adminUser
        )
      end
      NovellUtils.Nds2Ldap(adminUser)
    end

    # For use in non-dsfw mode or non-dsfw mode when joining a dsfw tree.
    def GetUserContext(adminUser)
      return "" if adminUser == nil || adminUser == ""

      userContext = adminUser
      Builtins.y2milestone("NovellLum.GetUserContext:user %1", userContext)

      # strip off any cn elements.
      while Builtins.issubstring(Builtins.tolower(userContext), "cn=")
        userContext = NovellUtils.GetContext(userContext)
      end
      Builtins.y2milestone("NovellLum.GetUserContext:context %1", userContext)
      userContext
    end

    def GetTopElement(name)
      name = NovellUtils.Nds2Ldap(name)
      tmp = Builtins.substring(name, Ops.add(Builtins.findlastof(name, ","), 1))
      tmp = name if tmp == nil || tmp == ""
      Builtins.y2milestone("novelllum.GetTopElement:returning %1", tmp)
      tmp
    end

    def GetFirstDCelement(name)
      name = NovellUtils.Nds2Ldap(name)
      split = Builtins.splitstring(Builtins.tolower(name), ",")

      returnStr = name
      Builtins.foreach(split) do |elem|
        if Builtins.issubstring(elem, "dc=")
          returnStr = elem
          raise Break
        end
      end
      Builtins.y2milestone(
        "novelllum.GetFirstDCelement:returning %1",
        returnStr
      )
      returnStr
    end


    def GetRootFrdContext(forestRootDomain)
      ret = ""
      cmd = Builtins.sformat(
        "/opt/novell/xad/share/dcinit/provisionTools.sh get-nc-for-lum -f '%1'",
        forestRootDomain
      )
      Builtins.y2milestone("NovellLum.GetRootFrdContext:executing: %1", cmd)
      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      if exit == 0
        ret = Ops.get_string(val, "stdout", "")
      elsif exit == 1
        Builtins.y2milestone(
          "NovellLum.GetRootFrdContext:required parameter missing."
        )
      elsif exit == 2
        Builtins.y2milestone(
          "NovellLum.GetRootFrdContext:server is not reachable with the given IP address."
        )
      else
        Builtins.y2milestone(
          "NovellLum.GetRootFrdContext:unknown error:%1",
          exit
        )
      end
      Builtins.y2milestone("NovellLum.GetRootFrdContext:returning: %1", ret)
      ret
    end


    # Calculate the XAD version of the Unix Config Object.
    # @return true on success
    def GetAndSetLumPartitionRoot
      ret = true

      domainType = GetDomainType()
      treeType = GetTreeType()
      serverContext = NovellUtils.Nds2Ldap(GetServerContext())
      forestRoot = GetForestRoot()
      Builtins.y2milestone(
        "NovellLum.GenerateXadUnixObjects:domainType %1, treeType %2, serverContext %3, forestRoot %4",
        domainType,
        treeType,
        serverContext,
        forestRoot
      )

      # First build the Unix config object context
      parentElement = ""
      if domainType == "forest"
        parentElement = serverContext
        Builtins.y2milestone(
          "NovellLum.GenerateXadUnixObjects:unix config for name-mapped and non-name-mapped FRD: %1",
          parentElement
        ) # child domain or additional domain controller
      else
        parentElement = GetRootFrdContext(forestRoot)
        Builtins.y2milestone(
          "NovellLum.GenerateXadUnixObjects:unix config for name-mapped and non-name-mapped child or DC is: %1",
          parentElement
        )
      end

      if parentElement == ""
        Builtins.y2milestone(
          "NovellLum.GenerateXadUnixObjects:could not determine Unix config object context. Falling back to secondary method."
        )
        if GetNameMap() == "yes"
          parentElement = GetTopElement(serverContext)
          Builtins.y2milestone(
            "NovellLum.GenerateXadUnixObjects:(fallback method) name mapped case, using top server context element: %1",
            parentElement
          )
        else
          # need to handle non-name map case.
          if treeType == "new" && domainType == "forest"
            Builtins.y2milestone(
              "NovellLum.GenerateXadUnixObjects:(fallback method) non name mapped FRD case, using server context: %1",
              serverContext
            )
            parentElement = serverContext
          else
            split = Builtins.splitstring(Builtins.tolower(forestRoot), ".")
            parentElement = Ops.add(
              Ops.add(@oesContext, ",dc="),
              Builtins.mergestring(split, ",dc=")
            ) # add the dc= elements
            Builtins.y2milestone(
              "NovellLum.GenerateXadUnixObjects:(fallback method) non name mapped case, using forest root: %1",
              parentElement
            )
          end
        end
      end
      parentElement = NovellUtils.CleanUpString(parentElement)
      Ops.set(@Settings, "CONFIG_LUM_PARTITION_ROOT", parentElement)
      Builtins.y2milestone(
        "NovellLum.GenerateXadUnixObjects:Unix Config Object Context:%1",
        Ops.get_string(@Settings, "CONFIG_LUM_PARTITION_ROOT", "")
      )
      ret = false if parentElement == nil || parentElement == ""

      ret
    end

    # GetAndSetWorkstationContext()
    #
    # Gets and sets the workstation context Settings value.
    #
    # returns true
    def GetAndSetWorkstationContext
      # this is the same for normal and XAD.
      wrkContext = GetServerContext()
      if wrkContext == nil || wrkContext == ""
        wrkContext = NovellUtils.GetContextNDS(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        Builtins.y2milestone(
          "NovellLum:Read.admin user context (could not get eDir server context): %1",
          wrkContext
        )
      end
      wrkContext = "" if wrkContext == nil || wrkContext == ""
      Ops.set(@Settings, "CONFIG_LUM_WS_CONTEXT", wrkContext)

      true
    end

    # GetAdminGroupContext()
    #
    # Calculates the admin group context for DSFW mode.
    #
    # returns true
    def GetAdminGroupContext
      groupContext = ""
      xadDomainName = GetXadDomainName()
      domainType = GetDomainType()
      if domainType == "controller" || domainType == "domain" # both of these could be non-name mapped on the box but could be joining a name mapped tree.
        serverCtx = GetServerContext()
        groupContext = ValidationHelpers.toLdap(serverCtx) # domainType -- "forest"
      else
        nameMap = GetNameMap()
        nameMapContext = GetNameMapContainer()
        if nameMap == "no"
          groupContext = Ops.add(@oesContext, ",")
          groupContext = Ops.add(
            groupContext,
            ValidationHelpers.toLdap(xadDomainName)
          )
        else
          groupContext = Ops.add(@oesContext, ",")
          groupContext = Ops.add(
            groupContext,
            ValidationHelpers.toLdap(nameMapContext)
          )
        end
      end
      Builtins.y2milestone(
        "NovellLum.GetAdminGroupContext:groupContext:%1",
        groupContext
      )
      NovellUtils.Nds2Ldap(groupContext)
    end

    # FixPamServiceConf( string file_name, list<string> add_list )
    #
    # puts the pam-nam entries in the service conf file.
    #
    # returns true if it is successful
    # returns false otherwise

    def FixPamServiceConf(file_name)
      Builtins.y2milestone(
        "-------------FixPamServiceConf started---------------"
      )
      retcode = false

      add_list = [
        "auth\tsufficient\tpam_nam.so\tuse_first_pass",
        "account\tsufficient\tpam_nam.so",
        "password\tsufficient\tpam_nam.so",
        "session\toptional\tpam_nam.so"
      ]

      target_dir = "/etc/pam.d/"
      data_file = Ops.add(target_dir, file_name)
      common_auth_file = Ops.add(target_dir, "common-auth")
      data_string = Convert.to_string(
        SCR.Read(path(".target.string"), data_file)
      )

      if data_string != nil && data_string != ""
        data_list = Builtins.splitstring(data_string, "\n")

        currlist = []
        beginlist = []
        endlist = []
        pos = 0
        at_end = false
        Builtins.foreach(data_list) do |val|
          if Builtins.regexpmatch(val, "^[\t ]*auth.*")
            currlist = Builtins.add(currlist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*account.*")
            currlist = Builtins.add(currlist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*password.*")
            currlist = Builtins.add(currlist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*session.*")
            currlist = Builtins.add(currlist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Ops.get(data_list, pos, "") != ""
            if at_end
              endlist = Builtins.add(endlist, Ops.get(data_list, pos, ""))
            else
              beginlist = Builtins.add(beginlist, Ops.get(data_list, pos, ""))
            end
          end
          pos = Ops.add(pos, 1)
        end
        #      y2milestone("testrun currlist-------------%1", currlist);
        #      y2milestone("testrun beginlist-------------%1", beginlist);
        #      y2milestone("testrun endlist-------------%1", endlist);


        outputstr = ""
        if Ops.greater_than(Builtins.size(beginlist), 0)
          Builtins.foreach(beginlist) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
          end
        end
        Builtins.foreach(add_list) do |val|
          outputstr = Ops.add(Ops.add(outputstr, val), "\n")
        end
        if Ops.greater_than(Builtins.size(currlist), 0)
          Builtins.foreach(currlist) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
          end
        end
        Builtins.foreach(endlist) do |val|
          outputstr = Ops.add(Ops.add(outputstr, val), "\n")
        end if Ops.greater_than(
          Builtins.size(endlist),
          0
        )
        #y2milestone("testrun outputstr-------------\n%1", outputstr);

        #SCR::Write(.target.string, data_file, "");
        if SCR.Write(path(".target.string"), data_file, outputstr)
          retcode = true
        end
      end

      Builtins.y2milestone(
        "-------------FixPamServiceConf ended ret %1-----------------",
        retcode
      )
      retcode
    end

    # FixPamServiceConf( string file_name, list<string> add_list )
    #
    # puts the pam-nam entries in the service conf file.
    #
    # returns true if it is successful
    # returns false otherwise
    def FixPamServiceConf_old(file_name)
      Builtins.y2milestone(
        "-------------FixPamServiceConf started---------------"
      )
      retcode = false

      add_list = [
        "auth\tsufficient\tpam_nam.so\tuse_first_pass",
        "account\tsufficient\tpam_nam.so",
        "password\tsufficient\tpam_nam.so",
        "session\toptional\tpam_nam.so"
      ]

      target_dir = "/etc/pam.d/"
      data_file = Ops.add(target_dir, file_name)
      common_auth_file = Ops.add(target_dir, "common-auth")
      data_string = Convert.to_string(
        SCR.Read(path(".target.string"), data_file)
      )

      if data_string != nil && data_string != ""
        data_list = Builtins.splitstring(data_string, "\n")
        common_auth_string = Convert.to_string(
          SCR.Read(path(".target.string"), common_auth_file)
        )
        common_auth_list = Builtins.splitstring(common_auth_string, "\n")
        # (#500184)	data_list = (list<string>)merge(data_list, add_list);

        ca_authreqlist = []
        ca_authlist = []
        ca_authunix2list = []
        ca_acclist = []
        ca_passlist = []
        ca_sesslist = []
        pos = 0
        Builtins.foreach(common_auth_list) do |val|
          if Builtins.regexpmatch(val, "^[\t ]*auth[ \t]+required")
            if Builtins.regexpmatch(
                val,
                "^[\t ]*auth[ \t]+required[ \t]+pam_unix2.so"
              )
              ca_authunix2list = Builtins.add(
                ca_authunix2list,
                Ops.get(common_auth_list, pos, "")
              )
            else
              ca_authreqlist = Builtins.add(
                ca_authreqlist,
                Ops.get(common_auth_list, pos, "")
              )
            end
          elsif Builtins.regexpmatch(val, "^[\t ]*auth.*")
            ca_authlist = Builtins.add(
              ca_authlist,
              Ops.get(common_auth_list, pos, "")
            )
          elsif Builtins.regexpmatch(val, "^[\t ]*account.*")
            ca_acclist = Builtins.add(
              ca_acclist,
              Ops.get(common_auth_list, pos, "")
            )
          elsif Builtins.regexpmatch(val, "^[\t ]*password.*")
            ca_passlist = Builtins.add(
              ca_passlist,
              Ops.get(common_auth_list, pos, "")
            )
          elsif Builtins.regexpmatch(val, "^[\t ]*session.*")
            ca_sesslist = Builtins.add(
              ca_sesslist,
              Ops.get(common_auth_list, pos, "")
            )
          end
          pos = Ops.add(pos, 1)
        end

        authreqlist = []
        authinclist = []
        authunix2list = []
        authlist = []
        acclist = []
        passlist = []
        sesslist = []
        beginlist = []
        endlist = []
        pos = 0
        at_end = false
        Builtins.foreach(data_list) do |val|
          if Builtins.regexpmatch(val, "^[\t ]*auth[ \t]+required")
            if Builtins.regexpmatch(
                val,
                "^[\t ]*auth[ \t]+required[ \t]+pam_unix2.so"
              )
              authunix2list = Builtins.add(
                authunix2list,
                Ops.get(data_list, pos, "")
              )
            else
              authreqlist = Builtins.add(
                authreqlist,
                Ops.get(data_list, pos, "")
              )
            end
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*auth[ \t]+include")
            if Builtins.regexpmatch(
                val,
                "^[\t ]*auth[ \t]+include[ \t]+common-auth"
              )
              if Ops.greater_than(Builtins.size(ca_authreqlist), 0)
                authreqlist = Convert.convert(
                  Builtins.merge(authreqlist, ca_authreqlist),
                  :from => "list",
                  :to   => "list <string>"
                )
              end
              if Ops.greater_than(Builtins.size(ca_authlist), 0)
                authlist = Convert.convert(
                  Builtins.merge(authlist, ca_authlist),
                  :from => "list",
                  :to   => "list <string>"
                )
              end
              if Ops.greater_than(Builtins.size(ca_authunix2list), 0)
                authunix2list = Convert.convert(
                  Builtins.merge(authunix2list, ca_authunix2list),
                  :from => "list",
                  :to   => "list <string>"
                )
              end
              if Ops.greater_than(Builtins.size(ca_acclist), 0)
                acclist = Convert.convert(
                  Builtins.merge(acclist, ca_acclist),
                  :from => "list",
                  :to   => "list <string>"
                )
              end
              if Ops.greater_than(Builtins.size(ca_passlist), 0)
                passlist = Convert.convert(
                  Builtins.merge(passlist, ca_passlist),
                  :from => "list",
                  :to   => "list <string>"
                )
              end
              if Ops.greater_than(Builtins.size(ca_sesslist), 0)
                sesslist = Convert.convert(
                  Builtins.merge(sesslist, ca_sesslist),
                  :from => "list",
                  :to   => "list <string>"
                )
              end
            else
              authinclist = Builtins.add(
                authinclist,
                Ops.get(data_list, pos, "")
              )
            end
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*auth.*")
            authlist = Builtins.add(authlist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*account.*")
            acclist = Builtins.add(acclist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*password.*")
            passlist = Builtins.add(passlist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Builtins.regexpmatch(val, "^[\t ]*session.*")
            sesslist = Builtins.add(sesslist, Ops.get(data_list, pos, ""))
            at_end = true
          elsif Ops.get(data_list, pos, "") != ""
            if at_end
              endlist = Builtins.add(endlist, Ops.get(data_list, pos, ""))
            else
              beginlist = Builtins.add(beginlist, Ops.get(data_list, pos, ""))
            end
          end
          pos = Ops.add(pos, 1)
        end
        #	y2milestone("testrun authreqlist-------------%1", authreqlist);
        #	y2milestone("testrun authinclist-------------%1", authinclist);
        #	y2milestone("testrun authunix2list-------------%1", authunix2list);
        #	y2milestone("testrun authlist-------------%1", authlist);
        #	y2milestone("testrun acclist-------------%1", acclist);
        #	y2milestone("testrun passlist-------------%1", passlist);
        #	y2milestone("testrun sesslist-------------%1", sesslist);
        #	y2milestone("testrun beginlist-------------%1", beginlist);
        #	y2milestone("testrun endlist-------------%1", endlist);


        outputstr = ""
        if Ops.greater_than(Builtins.size(beginlist), 0)
          Builtins.foreach(beginlist) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
          end
        end
        Builtins.foreach(add_list) do |val|
          outputstr = Ops.add(Ops.add(outputstr, val), "\n")
        end
        if Ops.greater_than(Builtins.size(authreqlist), 0)
          Builtins.foreach(authreqlist) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
          end
        end
        if Ops.greater_than(Builtins.size(authlist), 0)
          Builtins.foreach(authlist) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
          end
        end
        if Ops.greater_than(Builtins.size(authunix2list), 0)
          Builtins.foreach(ca_authunix2list) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
            raise Break
          end
        end
        Builtins.foreach(acclist) do |val|
          outputstr = Ops.add(Ops.add(outputstr, val), "\n")
        end if Ops.greater_than(
          Builtins.size(acclist),
          0
        )
        if Ops.greater_than(Builtins.size(passlist), 0)
          Builtins.foreach(passlist) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
          end
        end
        if Ops.greater_than(Builtins.size(sesslist), 0)
          Builtins.foreach(sesslist) do |val|
            outputstr = Ops.add(Ops.add(outputstr, val), "\n")
          end
        end
        Builtins.foreach(endlist) do |val|
          outputstr = Ops.add(Ops.add(outputstr, val), "\n")
        end if Ops.greater_than(
          Builtins.size(endlist),
          0
        )
        #	y2milestone("testrun outputstr-------------\n%1", outputstr);

        #SCR::Write(.target.string, data_file, "");
        if SCR.Write(path(".target.string"), data_file, outputstr)
          retcode = true
        end
      end

      Builtins.y2milestone(
        "-------------FixPamServiceConf ended ret %1-----------------",
        retcode
      )
      retcode
    end


    def UpdatePamServiceConf
      target_dir = "/etc/pam.d/"
      src = Ops.add(target_dir, "pam_nam_sample")

      login = Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_LOGIN", "no")
      ftp = Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_FTP", "no")
      sshd = Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SSHD", "no")
      su = Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SU", "no")
      sfcb = Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SFCB", "no")
      gdm = Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GDM", "no")
      gnome_screensaver = Ops.get_string(
        @Settings,
        "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER",
        "no"
      )

      gnomesu_pam = Ops.get_string(
        @Settings,
        "CONFIG_LUM_SERVICE_GNOMESU_PAM",
        "no"
      )


      listOfSelections = [
        login,
        ftp,
        su,
        sshd,
        sfcb,
        gdm,
        gnome_screensaver,
        gnomesu_pam
      ]
      listOfNames = [
        "login",
        "vsftpd",
        "su",
        "sshd",
        "sfcb",
        "gdm",
        "gnome-screensaver",
        "gnomesu-pam"
      ]

      # move the existing directory to a backup directory if the backup doesn't exist.
      execute_cmd = "test -d /etc/pam.d.nambkp"
      ret = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if ret != 0
        execute_cmd = "mkdir -p /etc/pam.d.nambkp"
        ret = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))

        execute_cmd = "cp /etc/pam.d/*  /etc/pam.d.nambkp"
        ret = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      end

      src_bkp = "/etc/pam.d.nambkp/"

      Builtins.foreach([0, 1, 2, 3, 4, 5, 6, 7]) do |i|
        raise Break if @xadTestMode
        # verify if the file already has the entries for PAM
        file = Ops.get(listOfNames, i, "tmp")
        if SCR.Read(path(".target.size"), Ops.add(target_dir, file)) != -1
          execute_string1 = Ops.add(
            Ops.add(Ops.add(Ops.add("cat ", target_dir), file), " | "),
            "grep pam_nam.so > /dev/null"
          )
          Builtins.y2debug("LUM CMD: %1", execute_string1)
          ret = Convert.to_integer(
            SCR.Execute(path(".target.bash"), execute_string1)
          )
        end
        ans = Ops.get(listOfSelections, i, "no")
        if ans == "yes"
          # verify if the file already has the entries for PAM
          file2 = Ops.get(listOfNames, i, "tmp")
          if SCR.Read(path(".target.size"), Ops.add(target_dir, file2)) != -1
            execute_string1 = Ops.add(
              Ops.add(Ops.add(Ops.add("cat ", target_dir), file2), " | "),
              "grep pam_nam.so > /dev/null"
            )
            Builtins.y2debug("LUM CMD: %1", execute_string1)
            ret = Convert.to_integer(
              SCR.Execute(path(".target.bash"), execute_string1)
            )
            if ret != 0
              # this service is not pam_enabled
              FixPamServiceConf(file2) 
              #string execute_string2 = "sed r " + src + " " + src_bkp + file + " | "+ "tee " +  target_dir + file + "> /dev/null";
              #y2debug("LUM CMD: %1",execute_string2);
              #ret = (integer)SCR::Execute(.target.bash, execute_string2);
            end
          end
        elsif ans == "no"
          # verify if the file already has the entries for PAM
          file2 = Ops.get(listOfNames, i, "tmp")
          if SCR.Read(path(".target.size"), Ops.add(target_dir, file2)) != -1
            execute_string1 = Ops.add(
              Ops.add(Ops.add(Ops.add("cat ", target_dir), file2), " | "),
              "grep pam_nam.so > /dev/null"
            )
            Builtins.y2debug("LUM CMD: %1", execute_string1)
            ret = Convert.to_integer(
              SCR.Execute(path(".target.bash"), execute_string1)
            )
            if ret == 0
              # this service is pam_enabled. Remove the pam_name.so entries.
              execute_cmd = Builtins.sformat(
                "sed -i 's/.*pam_nam.so.*//' /etc/pam.d/%1",
                file2
              )
              ret = Convert.to_integer(
                SCR.Execute(path(".target.bash"), execute_cmd)
              ) 

              # this service is pam_enabled. Restore the file from backup dir.
              #execute_cmd = sformat ("cp /etc/pam.d.nambkp/%1  /etc/pam.d/%1", file);
              #ret = (integer)SCR::Execute(.target.bash, execute_cmd);
            end
          end
        end
      end

      nil
    end


    # Read all novell-lum settings
    # @return true on success
    def Read
      # NovellLum read dialog caption
      caption = _("Initializing Linux User Management")

      steps = 2

      sl = 500
      Builtins.sleep(sl)


      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/3
          _("Read the database"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/3
          _("Reading the database..."),
          _("Checking dependencies"),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read database
      return false if Abort()
      Progress.NextStage

      Builtins.y2milestone("NovellLum:Entering Read()")

      # For the upgrade scenario
      @upgrade_from = NovellUtils.UpgradeIsFrom
      if (Mode.update || Mode.autoupgrade) &&  @upgrade_from != ""
        @oldSettings = NovellUtils.GetOldSysconfigMap("novell-lum")
      end

      Builtins.y2milestone(
        "novell-lum: 
        upgradeFrom = #{@upgrade_from}"
      )
      Builtins.y2milestone("novell-lum:oldSettings: %1", @oldSettings)

      info = NovellAddonInstall.GetProductInfo("edirectory")
      if Ops.get_boolean(info, "SERVICE_CONFIGURED", false) == true
        @edir_service_configured = true
      else
        @edir_service_configured = false
      end

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      setDefaultIP = false

      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "NovellLum:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
        Builtins.y2milestone(
          "NovellLum:xad configured from eDir map: %1",
          Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "")
        )
        Builtins.y2milestone(
          "NovellLum:xad configured from file: %1",
          Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
          )
        )
        if Mode.update || Mode.autoupgrade
          @xadMode = true
        end
      end

      Builtins.y2milestone("NovellLum:xadMode:%1", @xadMode)

      if @xadMode
        change_oes_context = Builtins.tolower(
          NovellUtils.ReadOption("CHANGED_OES_CONTEXT")
        )
        Builtins.y2milestone(
          "NovellLum:dsfw context changed is %1",
          change_oes_context
        )
        @oesContext = "ou=novell" if change_oes_context == "yes"
      end

      # If we are upgrading from DSFW then the Edirectory map will not contain name map data since it is not used in an upgrade. We must get this data from the old config files.
      if (Mode.update || Mode.autoupgrade) && @xadMode

        @old_EdirSettings = NovellUtils.GetOldSysconfigMap("edirectory")
        @old_XadSettings = NovellUtils.GetOldSysconfigMap("xad")
        Builtins.y2milestone(
          "lum:OES  edir oldsettings: %1",
          @old_EdirSettings
        )
        Builtins.y2milestone(
          "lum:OES xad oldsettings: %1",
          @old_XadSettings
        )

        @doRecalculate = false # don't recalculate the ws context in Write() since we are getting the value from the old sysconfig file.
      end

      # determine if LUM configuration already exists - if it does not, do the following...
      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.novell-lum.SERVICE_CONFIGURED"))
      )
      service_configured = "no" if service_configured == nil
      service_configured = Builtins.tolower(service_configured)
      if Mode.update || Mode.autoupgrade
        Ops.set(
          @Settings,
          "CONFIG_LUM_LDAP_SERVER",
          Ops.get_string(@oldSettings, "CONFIG_LUM_LDAP_SERVER", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_WS_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_LUM_WS_CONTEXT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_PARTITION_ROOT",
          Ops.get_string(@oldSettings, "CONFIG_LUM_PARTITION_ROOT", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_PROXY_USER",
          Ops.get_string(@oldSettings, "CONFIG_LUM_PROXY_USER", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_RESTRICT_ACCESS",
          Ops.get_string(@oldSettings, "CONFIG_LUM_RESTRICT_ACCESS", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_SERVICE_LOGIN",
          Ops.get_string(@oldSettings, "CONFIG_LUM_SERVICE_LOGIN", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_SERVICE_FTP",
          Ops.get_string(@oldSettings, "CONFIG_LUM_SERVICE_FTP", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_SERVICE_SSHD",
          Ops.get_string(@oldSettings, "CONFIG_LUM_SERVICE_SSHD", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_SERVICE_SU",
          Ops.get_string(@oldSettings, "CONFIG_LUM_SERVICE_SU", "")
        )
        Ops.set(@Settings, "CONFIG_LUM_SERVICE_SFCB", "yes")
        Ops.set(
          @Settings,
          "CONFIG_LUM_SERVICE_GDM",
          Ops.get_string(@oldSettings, "CONFIG_LUM_SERVICE_GDM", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER",
          Ops.get_string(
            @oldSettings,
            "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER",
            ""
          )
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_SERVICE_GNOMESU_PAM",
          Ops.get_string(@oldSettings, "CONFIG_LUM_SERVICE_GNOMESU_PAM", "")
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_ADMIN_GROUP",
          Ops.get_string(@oldSettings, "CONFIG_LUM_ADMIN_GROUP", "")
        )
        list1 = Ops.get_string(
          @oldSettings,
          "CONFIG_LUM_ALTERNATE_SERVERS_LIST1",
          ""
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_ALTERNATE_SERVERS_LIST1",
          Builtins.splitstring(list1, ",")
        )
        list2 = Ops.get_string(
          @oldSettings,
          "CONFIG_LUM_ALTERNATE_SERVERS_LIST2",
          ""
        )
        Ops.set(
          @Settings,
          "CONFIG_LUM_ALTERNATE_SERVERS_LIST2",
          Builtins.splitstring(list2, ",")
        )
        if Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "") == ""
          admin_fdn = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
          admin_context = NovellUtils.GetContextNDS(admin_fdn)

          # Calculate the default value for admin group.
          wrkAdminGroup = Ops.add(
            "cn=admingroup,",
            NovellUtils.Nds2Ldap(admin_context)
          )
          if @xadMode
            wrkAdminGroup = Ops.add("cn=admingroup,", GetAdminGroupContext())
          end
          Ops.set(@Settings, "CONFIG_LUM_ADMIN_GROUP", wrkAdminGroup)
        end
        Builtins.y2milestone(
          "CONFIG_LUM_ADMIN_GROUP value is set to %1",
          Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
        )
        @doRecalculate = false # don't recalculate the ws context in Write() since we are getting the value from the old sysconfig file. # normal non-upgrade install mode
      else
        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.Read:useCommonProxy: %1",
          useCommonProxy
        )

        commonProxyName = ""
        if useCommonProxy == "yes"
          commonProxyName = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
          Builtins.y2milestone(
            "NovellLum.Read:commonProxyName: '%1'",
            commonProxyName
          )
          commonProxyPassword = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_PROXY_PASSWORD",
            ""
          )
        end

        if service_configured != "yes"
          setDefaultIP = true

          # Now, set up the data.;
          # LUM wants the remote server if availble
          server_list = NovellLDAP.GetRemoteServers(nil)
          if Ops.greater_than(Builtins.size(server_list), 0)
            Ops.set(
              @Settings,
              "CONFIG_LUM_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          else
            server_list = NovellLDAP.GetLocalServers(nil)
            Ops.set(
              @Settings,
              "CONFIG_LUM_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          end
          server_list = Ops.get_list(
            NovellLDAP.Settings,
            "CONFIG_LDAP_SERVERS_LIST",
            []
          )
          alternate_ldap_servers_list1 = Ops.get_list(
            @Settings,
            "CONFIG_LUM_ALTERNATE_SERVERS_LIST1",
            []
          )
          alternate_ldap_servers_list2 = Ops.get_list(
            @Settings,
            "CONFIG_LUM_ALTERNATE_SERVERS_LIST2",
            []
          )
          ldap_server = Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", "")
          alternate_ldap_server_list = []
          Builtins.foreach(server_list) do |server_info|
            if Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "") != ldap_server
              alternate_ldap_server_list = Builtins.add(
                alternate_ldap_server_list,
                server_info
              )
            end
          end 

          Ops.set(
            @Settings,
            "CONFIG_LUM_LDAP_SERVERS",
            alternate_ldap_server_list
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_ALTERNATE_SERVERS_LIST1",
            alternate_ldap_servers_list1
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_ALTERNATE_SERVERS_LIST2",
            alternate_ldap_servers_list2
          )


          domainName = ""
          xadContext = ""

          bVal = GetAndSetWorkstationContext()

          if @xadMode
            bVal = GetAndSetLumPartitionRoot()
          else
            Ops.set(
              @Settings,
              "CONFIG_LUM_PARTITION_ROOT",
              GetUserContext(
                Ops.get_string(
                  NovellLDAP.Settings,
                  "CONFIG_LDAP_ADMIN_CONTEXT",
                  ""
                )
              )
            )
          end

          Ops.set(@Settings, "CONFIG_LUM_PROXY_USER", "")
          Ops.set(@Settings, "CONFIG_LUM_RESTRICT_ACCESS", "yes")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_LOGIN", "no")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_FTP", "no")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_SSHD", "no")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_SU", "no")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_SFCB", "yes")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_GDM", "no")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER", "no")
          Ops.set(@Settings, "CONFIG_LUM_SERVICE_GNOMESU_PAM", "no")

          admin_fdn = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
          admin_context = NovellUtils.GetContextNDS(admin_fdn)

          # Calculate the default value for admin group.
          wrkAdminGroup = Ops.add(
            "cn=admingroup,",
            NovellUtils.Nds2Ldap(admin_context)
          )
          if @xadMode
            wrkAdminGroup = Ops.add("cn=admingroup,", GetAdminGroupContext())
          end
          Ops.set(@Settings, "CONFIG_LUM_ADMIN_GROUP", wrkAdminGroup)
          Builtins.y2milestone(
            "CONFIG_LUM_ADMIN_GROUP value is set to %1",
            Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
          )

          if useCommonProxy == "yes"
            Ops.set(@Settings, "CONFIG_LUM_COMMON_PROXY", commonProxyName)

            if commonProxyPassword != ""
              Ops.set(
                @Settings,
                "CONFIG_LUM_COMMON_PROXY_PASSWORD",
                commonProxyPassword
              ) # new install
            else
              Ops.set(
                @Settings,
                "CONFIG_LUM_COMMON_PROXY_PASSWORD",
                NovellUtils.RetrieveProxyCredentials("password")
              )
            end # post install
          end # We have our own settings, so we will use them. Post install reconfigure mode.
        else
          @doRecalculate = false # don't recalculate the ws context in Write() since we are getting the value from the sysconfig file.

          Ops.set(
            @Settings,
            "CONFIG_LUM_LDAP_SERVER",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_LDAP_SERVER"),
              ""
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_WS_CONTEXT",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_WS_CONTEXT"),
              ""
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_PARTITION_ROOT",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_PARTITION_ROOT"),
              ""
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_PROXY_USER",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_PROXY_USER"),
              ""
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_RESTRICT_ACCESS",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_RESTRICT_ACCESS"),
              ""
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_LOGIN",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_LOGIN"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_FTP",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_FTP"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_SSHD",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SSHD"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_SU",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SU"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_SFCB",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SFCB"),
              "yes"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_GDM",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GDM"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GNOME_SCREENSAVER"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_SERVICE_GNOMESU_PAM",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GNOMESU_PAM"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_ADMIN_GROUP",
            Misc.SysconfigRead(
              path(".sysconfig.novell-lum.CONFIG_LUM_ADMIN_GROUP"),
              "no"
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_ALTERNATE_SERVERS_LIST1",
            Builtins.splitstring(
              Misc.SysconfigRead(
                path(".sysconfig.novell-lum.CONFIG_LUM_ALTERNATE_SERVERS_LIST1"),
                ""
              ),
              ","
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_LUM_ALTERNATE_SERVERS_LIST2",
            Builtins.splitstring(
              Misc.SysconfigRead(
                path(".sysconfig.novell-lum.CONFIG_LUM_ALTERNATE_SERVERS_LIST2"),
                ""
              ),
              ","
            )
          )

          if useCommonProxy == "yes"
            if commonProxyPassword == ""
              commonProxyPassword = NovellUtils.RetrieveProxyCredentials(
                "password"
              )
            end

            if NovellUtils.Nds2Ldap(
                Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
              ) ==
                NovellUtils.Nds2Ldap(commonProxyName)
              Ops.set(
                @Settings,
                "CONFIG_LUM_PROXY_USER_PASSWORD",
                commonProxyPassword
              )
            end
            Ops.set(@Settings, "CONFIG_LUM_COMMON_PROXY", commonProxyName)
            Ops.set(
              @Settings,
              "CONFIG_LUM_COMMON_PROXY_PASSWORD",
              commonProxyPassword
            )
          end
          if !@lum_proxy
            cmd = "/usr/bin/lum_retrieve_proxy_cred password /tmp/lum.txt"
            val = Convert.to_integer(SCR.Execute(path(".target.bash"), cmd))
            if val != 0
              Builtins.y2milestone(
                "Lum proxy password fectching from OCS failed"
              )
            else
              password = Convert.to_string(
                SCR.Read(path(".target.string"), "/tmp/lum.txt")
              )
            end
            val = Convert.to_integer(
              SCR.Execute(path(".target.bash_output"), "rm -rf /tmp/lum.txt")
            )
            Ops.set(@Settings, "CONFIG_LUM_PROXY_USER_PASSWORD", password.nil? ? "" : password)
          end
        end
      end

      if Mode.update || Mode.autoupgrade
        Builtins.y2milestone("Updating the pam service conf files")
        UpdatePamServiceConf()
      end

      Builtins.y2debug("NovellLum:READ Complete:%1", @Settings)

      # Error message
      Report.Error(_("Cannot read the database1.")) if false
      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      @globals_configured = true

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end


    # In xad mode server context can look like "ou=OESSystemObjects.ou=blocker.o=novell" or "ou=OESSystemObjects.dc=blocker.dc=com"
    # This method stripts the ou=OESSystemObjects off of the context
    def GetNameContextFromServerContext(serverContext)
      Builtins.y2milestone(
        "NovellLum.GetNameContextFromServerContext:server context = %1",
        serverContext
      )
      serverContext = NovellUtils.Nds2Ldap(serverContext)
      elementToRemove = @oesContext

      idx = Builtins.search(serverContext, elementToRemove)
      if idx != nil
        if idx == 0
          serverContext = Builtins.substring(
            serverContext,
            Ops.add(Builtins.size(elementToRemove), 1),
            Builtins.size(serverContext)
          )
        end
      end
      Builtins.y2milestone(
        "NovellLum.GetNameContextFromServerContext:name context = %1",
        serverContext
      )
      serverContext
    end

    # Write all novell-lum settings
    # @return true on success
    def Write
      Wizard.DisableAbortButton
      Builtins.y2milestone("NovellLum::Write started")
      pattern = "novell-lum"
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end

      if !Ops.get(NovellAddonInstall.products_to_configure, "novell-lum", true)
        Builtins.y2milestone("Linux User Management -- configure later")
        return true
      end

      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone(
          "NovellLum:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        return true
      end

      if Mode.autoinst
        commonProxyName = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
        )
        if commonProxyName != ""
          Ops.set(@Settings, "CONFIG_LUM_PROXY_USER", commonProxyName)
        else
          Ops.set(@Settings, "CONFIG_LUM_PROXY_USER", NovellUtils.RetrieveProxyCredentials("username"))
        end
        commonProxyPassword = Ops.get_string(NovellLDAP.Settings,"CONFIG_LDAP_PROXY_PASSWORD","")
        if commonProxyPassword != ""
          Ops.set(@Settings, "CONFIG_LUM_PROXY_USER_PASSWORD", commonProxyPassword)
        else
          Ops.set(@Settings, "CONFIG_LUM_PROXY_USER_PASSWORD", NovellUtils.RetrieveProxyCredentials("password"))
        end
      end

      # If XAD did not install properly, then we don't want to proceed at this point.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "NovellLum:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
        Builtins.y2milestone(
          "NovellLum:xad configured from eDir map: %1",
          Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "")
        )
        Builtins.y2milestone(
          "NovellLum:xad configured from file: %1",
          Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
          )
        )
      end
      Builtins.y2milestone("NovellLum:xadMode:%1", @xadMode)

      Builtins.y2milestone(
        "NovellLum:testing to see if eDir configured properly. Xad mode: %1",
        @xadMode
      )
      if @xadMode == true
        Builtins.y2milestone(
          "NovellLum:eDir service_configured (mem):%1",
          Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "")
        )
        Builtins.y2milestone(
          "NovellLum:eDir service_configured (file):%1",
          Convert.to_string(
            SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
          )
        )
        if Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "") != "yes" &&
            Convert.to_string(
              SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
            ) != "yes"
          if EdirUtils.EdirConfigProblemPrompt("Linux User Management")
            Builtins.y2milestone(
              "NovellLum:aborting LUM write by user choice since eDirectory was not configured."
            )
            return true
          end
        end
      end

      # NovellLum write dialog caption
      caption = _("Saving Linux User Management Configuration")

      # TODO FIXME And set the right number of stages
      steps = 1

      sl = 500
      Builtins.sleep(sl)


      # TODO FIXME Names of real stages
      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Write the settings")
        ],
        [
          # Progress step 1/1
          _("Writing the settings..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # write settings
      return false if Abort()
      Progress.NextStage

      upgradeStr = ""
      if Mode.update || Mode.autoupgrade
        upgradeStr = "UPGRADE='1' "
      end

      if  !Mode.update || !Mode.autoupgrade
        # we skip config on the last two version upgrades
        # Since eDir may have failed during Write() and the user may have changed the server context to fix that issue, we will recalculate values based on eDir context.
        # The original eDir context obtained during Read would no longer be valid (#628274).
        if @userChangedData == false
          GetAndSetWorkstationContext() if @doRecalculate
        end

        admin_fdn = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        pwd = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )

        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "NovellLum:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            admin_fdn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
            pwd = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
          end
        end
        Builtins.y2milestone("NovellLum:admin:%1", admin_fdn)

        partition = Ops.get_string(@Settings, "CONFIG_LUM_PARTITION_ROOT", "")
        ws_ctx = Ops.get_string(@Settings, "CONFIG_LUM_WS_CONTEXT", "")
        if Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "") == ""
          admin_fdn2 = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
          )
          admin_context = NovellUtils.GetContextNDS(admin_fdn2)

          # Calculate the default value for admin group.
          wrkAdminGroup = Ops.add(
            "cn=admingroup,",
            NovellUtils.Nds2Ldap(admin_context)
          )
          if @xadMode
            wrkAdminGroup = Ops.add("cn=admingroup,", GetAdminGroupContext())
          end
          Ops.set(@Settings, "CONFIG_LUM_ADMIN_GROUP", wrkAdminGroup)
          Builtins.y2milestone(
            "CONFIG_LUM_ADMIN_GROUP value is set to %1",
            Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
          )
        end
        alternate_ldap_servers_list1 = Ops.get_list(
          @Settings,
          "CONFIG_LUM_ALTERNATE_SERVERS_LIST1",
          []
        )

        alternate_ldap_servers_list1_content = ""
        Builtins.foreach(alternate_ldap_servers_list1) do |alternate_ldap_server|
          alternate_ldap_servers_list1_content = Ops.add(
            Ops.add(",", alternate_ldap_server),
            alternate_ldap_servers_list1_content
          )
        end
        alternate_ldap_servers_list1_content = Builtins.substring(
          alternate_ldap_servers_list1_content,
          1
        )

        alternate_ldap_servers_list2 = Ops.get_list(
          @Settings,
          "CONFIG_LUM_ALTERNATE_SERVERS_LIST2",
          []
        )
        alternate_ldap_servers_list2_content = ""
        Builtins.foreach(alternate_ldap_servers_list2) do |alternate_ldap_server|
          alternate_ldap_servers_list2_content = Ops.add(
            Ops.add(",", alternate_ldap_server),
            alternate_ldap_servers_list2_content
          )
        end
        alternate_ldap_servers_list2_content = Builtins.substring(
          alternate_ldap_servers_list2_content,
          1
        )


        SCR.Write(path(".sysconfig.novell-lum.SERVICE_CONFIGURED"), "no")
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_LDAP_SERVER"),
          Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_PARTITION_ROOT"),
          Ops.get_string(@Settings, "CONFIG_LUM_PARTITION_ROOT", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_WS_CONTEXT"),
          Ops.get_string(@Settings, "CONFIG_LUM_WS_CONTEXT", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_PROXY_USER"),
          Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_RESTRICT_ACCESS"),
          Ops.get_string(@Settings, "CONFIG_LUM_RESTRICT_ACCESS", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_LOGIN"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_LOGIN", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_FTP"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_FTP", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SSHD"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SSHD", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SU"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SU", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SFCB"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SFCB", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GDM"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GDM", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GNOME_SCREENSAVER"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GNOMESU_PAM"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GNOMESU_PAM", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_ADMIN_GROUP"),
          Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_ALTERNATE_SERVERS_LIST1"),
          Builtins.mergestring(
            Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST1", []),
            ","
          )
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_ALTERNATE_SERVERS_LIST2"),
          Builtins.mergestring(
            Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST2", []),
            ","
          )
        )

        # Flush file
        SCR.Write(path(".sysconfig.novell-lum"), nil)

        # determine IP addresses
        server_list = Ops.get_list(
          NovellLDAP.Settings,
          "CONFIG_LDAP_SERVERS_LIST",
          []
        )
        primary_server = NovellLDAP.GetAddressServer(
          Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", ""),
          server_list
        )

        # bugzilla 392899
        if primary_server == nil || primary_server == {}
          Builtins.y2milestone(
            "NovellLum:primary server was empty. Trying again."
          )
          server_list2 = NovellLDAP.GetRemoteServers(nil)
          if Ops.greater_than(Builtins.size(server_list2), 0)
            Ops.set(
              @Settings,
              "CONFIG_LUM_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list2, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          else
            Builtins.y2milestone("NovellLum:no remote servers, using local ip.")
            server_list2 = NovellLDAP.GetLocalServers(nil)
            Ops.set(
              @Settings,
              "CONFIG_LUM_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list2, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          end

          primary_server = NovellLDAP.GetAddressServer(
            Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", ""),
            server_list2
          )
        end

        fallback_list = NovellLDAP.GetFallbackServerList(
          Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", ""),
          server_list
        )
        Builtins.y2milestone("Primary server: %1", primary_server)
        Builtins.y2milestone("Fallback server_list: %1", fallback_list)

        proxy_user2 = Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
        proxy_user2 = "" if proxy_user2 == nil
        proxy_pwd = Ops.get_string(
          @Settings,
          "CONFIG_LUM_PROXY_USER_PASSWORD",
          ""
        )
        proxy_pwd = "" if proxy_pwd == nil

        useCommonProxy = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_DO_PROXY",
          ""
        )
        Builtins.y2milestone(
          "NovellLum.Write:useCommonProxy: %1",
          useCommonProxy
        )
        commonProxyName = ""
        if useCommonProxy == "yes"
          commonProxyName = NovellUtils.Nds2Ldap(
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", "")
          )
          commonProxyPassword = NovellUtils.RetrieveProxyCredentials("password")
          if commonProxyPassword == nil || commonProxyPassword == ""
            commonProxyPassword = Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_PROXY_PASSWORD",
              ""
            )
          end

          if NovellUtils.Nds2Ldap(
              Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
            ) ==
              NovellUtils.Nds2Ldap(commonProxyName)
            proxy_pwd = commonProxyPassword
            Ops.set(@Settings, "CONFIG_LUM_PROXY_USER_PASSWORD", proxy_pwd)
          end
        end

        execute_cmd = ""
        simple = false
        if !NovellUtils.IsServerProduct
          if admin_fdn == nil
            admin_fdn = ""
            simple = true
          end
        end

        # To get around an ldif schema extension issue when joining an old tree we extend nam.sch in advace here.
        full_admin_context = NovellUtils.Ldap2Nds(admin_fdn)
        Builtins.y2milestone("NovellLum:admin:%1", full_admin_context)

        schema_file = "/opt/novell/sch/NAM.SCH"
        ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
        cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(ndssch_file, " -h "),
                        Ops.get_string(
                          primary_server,
                          "CONFIG_LDAP_SERVER_IP",
                          ""
                        )
                      ),
                      " -F '"
                    ),
                    @schemaLogPath
                  ),
                  "' '"
                ),
                full_admin_context
              ),
              "'"
            ),
            " "
          ),
          schema_file
        )
        if @xadTestMode
          Builtins.y2milestone(
            "NovellLum:Executing %1",
            Ops.add(Ops.add(Ops.add("ADM_PASSWD='", pwd), "' "), cmd)
          )
        else
          Builtins.y2milestone("NovellLum:Executing %1", cmd)
        end
        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(Ops.add(Ops.add("ADM_PASSWD='", pwd), "' "), cmd)
          )
        )
        if Ops.get_integer(rmap, "exit", -1) != 0
          Builtins.foreach(fallback_list) do |server_info2|
            cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(ndssch_file, " -h "),
                            Ops.get_string(
                              server_info2,
                              "CONFIG_LDAP_SERVER_IP",
                              ""
                            )
                          ),
                          " -F '"
                        ),
                        @schemaLogPath
                      ),
                      "' '"
                    ),
                    full_admin_context
                  ),
                  "'"
                ),
                " "
              ),
              schema_file
            )
            if @xadTestMode
              Builtins.y2milestone(
                "xad:NovellLum:Retry of schema extension with fallback IP address / hostname:Executing %1",
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", pwd), "' "), cmd)
              )
            else
              Builtins.y2milestone(
                "NovellLum:Retry of schema extension with fallback IP address / hostname:Executing %1",
                cmd
              )
            end
            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", pwd), "' "), cmd)
              )
            )
            if Ops.get_integer(rmap, "exit", -1) != 0
              Builtins.y2milestone(
                "NovellLum:extend schema(sch) failed: %1 using file %2",
                Ops.get_integer(rmap, "exit", -1),
                schema_file
              )
            else
              raise Break
            end
          end
        end

        ip = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
        port = Ops.get_integer(primary_server, "CONFIG_LDAP_PORT", 389)
        sPort = Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)

        server_info = Ops.get(fallback_list, 0, {})
        if @xadMode
          ip = Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
          port = Ops.get_integer(server_info, "CONFIG_LDAP_PORT", 389)
          sPort = Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636)
          if ip == nil || ip == ""
            ip = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
            port = Ops.get_integer(primary_server, "CONFIG_LDAP_PORT", 389)
            sPort = Ops.get_integer(
              primary_server,
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
          end
        end

        restrict_access = Ops.get_string(
          @Settings,
          "CONFIG_LUM_RESTRICT_ACCESS",
          ""
        )
        cmd_server_params = " -r '%1' -S %2:%3 -l %4"
        if simple
          execute_cmd = Builtins.sformat(
            Ops.add("namconfig simple_add", cmd_server_params),
            ValidationHelpers.toLdap(partition),
            ip,
            port,
            sPort
          )
        else
          if restrict_access == "yes"
            execute_cmd = Builtins.sformat(
              Ops.add("namconfig add -u -a '%5' -w '%6'", cmd_server_params),
              ValidationHelpers.toLdap(partition),
              ip,
              port,
              sPort,
              ValidationHelpers.toLdap(admin_fdn),
              ValidationHelpers.toLdap(ws_ctx)
            )
          else
            execute_cmd = Builtins.sformat(
              Ops.add("namconfig add -a '%5' -w '%6'", cmd_server_params),
              ValidationHelpers.toLdap(partition),
              ip,
              port,
              sPort,
              ValidationHelpers.toLdap(admin_fdn),
              ValidationHelpers.toLdap(ws_ctx)
            )
          end
        end
        logCmd = execute_cmd
        if proxy_user2 != ""
          execute_cmd = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(execute_cmd, " -y '"),
                      ValidationHelpers.toLdap(proxy_user2)
                    ),
                    "'"
                  ),
                  " -d "
                ),
                "'"
              ),
              proxy_pwd
            ),
            "'"
          )


          logCmd = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(logCmd, " -y '"),
                  ValidationHelpers.toLdap(proxy_user2)
                ),
                "'"
              ),
              " -d "
            ),
            "****"
          )
        end
        if Ops.greater_than(
            Builtins.size(alternate_ldap_servers_list1_content),
            0
          )
          execute_cmd = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(execute_cmd, " -y '"),
                  ValidationHelpers.toLdap(proxy_user2)
                ),
                "'"
              ),
              " -R "
            ),
            alternate_ldap_servers_list1_content
          )


          logCmd = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(logCmd, " -y '"),
                  ValidationHelpers.toLdap(proxy_user2)
                ),
                "'"
              ),
              " -R "
            ),
            alternate_ldap_servers_list1_content
          )
        end
        if Ops.greater_than(
            Builtins.size(
              Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST2", [])
            ),
            0
          )
          if Ops.greater_than(
              Builtins.size(alternate_ldap_servers_list1_content),
              0
            )
            execute_cmd = Ops.add(
              Ops.add(execute_cmd, ","),
              alternate_ldap_servers_list2_content
            )
            logCmd = Ops.add(
              Ops.add(logCmd, ","),
              alternate_ldap_servers_list2_content
            )
          else
            execute_cmd = Ops.add(
              Ops.add(execute_cmd, " -R "),
              alternate_ldap_servers_list2_content
            )
            logCmd = Ops.add(
              Ops.add(logCmd, " -R "),
              alternate_ldap_servers_list2_content
            )
          end
        end
        execute_cmd = Ops.add(execute_cmd, " 2>&1")
        logCmd = Ops.add(logCmd, " 2>&1")

        if !simple
          execute_cmd = Ops.add(
            Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
            execute_cmd
          )
        end

        Builtins.y2debug("LUM Apply Settings, cmd = %1", execute_cmd)

        Popup.ShowFeedback(
          _("Please Wait"),
          _("Running namconfig for LUM, this may take a while...")
        )
        ret = 0
        Builtins.foreach([1, 2, 3]) do |i|
          ret = 0
          if @xadTestMode == false
            ret = Convert.to_integer(
              SCR.Execute(
                path(".target.bash"),
                Ops.add(upgradeStr, execute_cmd)
              )
            )
          end
          if @xadTestMode
            Builtins.y2milestone(
              "NovellLum:execute command: %1",
              Ops.add(upgradeStr, logCmd)
            )
          else
            Builtins.y2milestone("NovellLum:execute command: %1", logCmd)
          end
          raise Break if ret == 0
          raise Break if @xadTestMode
          Builtins.y2milestone("NovellLum:sleeping 5 seconds.")
          Builtins.sleep(5000) #sleep 15 seconds
        end

        if ret != 0
          Builtins.y2milestone(
            "NovellLum:retry loop timed out with no success."
          )
          if @xadMode == false
            ip = Ops.get_string(server_info, "CONFIG_LDAP_SERVER_IP", "")
            port = Ops.get_integer(server_info, "CONFIG_LDAP_PORT", 389)
            sPort = Ops.get_integer(server_info, "CONFIG_LDAP_SECURE_PORT", 636)
          else
            ip = Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", "")
            port2 = Ops.get_integer(primary_server, "CONFIG_LDAP_PORT", 389)
            sPort2 = Ops.get_integer(
              primary_server,
              "CONFIG_LDAP_SECURE_PORT",
              636
            )
          end

          # fall back to the other servers if the primary server failed
          Builtins.foreach(fallback_list) do |server_info2|
            Builtins.y2milestone(
              "namconfig failed - trying %1",
              Ops.get_string(server_info2, "CONFIG_LDAP_SERVER_IP", "")
            )
            execute_cmd = Builtins.sformat(
              Ops.add("namconfig add -a '%5' -w '%6'", cmd_server_params),
              ValidationHelpers.toLdap(partition),
              ip,
              port,
              sPort,
              ValidationHelpers.toLdap(admin_fdn),
              ValidationHelpers.toLdap(ws_ctx)
            )
            logCmd2 = execute_cmd
            if proxy_user2 != ""
              execute_cmd = Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(execute_cmd, " -y '"),
                          ValidationHelpers.toLdap(proxy_user2)
                        ),
                        "'"
                      ),
                      " -d "
                    ),
                    "'"
                  ),
                  proxy_pwd
                ),
                "'"
              )

              logCmd2 = Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(logCmd2, " -y '"),
                      ValidationHelpers.toLdap(proxy_user2)
                    ),
                    "'"
                  ),
                  " -d "
                ),
                "****"
              )
            end
            execute_cmd = Ops.add(execute_cmd, " 2>&1")
            Builtins.y2milestone("NovellLum:execute command: %1", logCmd2)
            execute_cmd = Ops.add(
              Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
              execute_cmd
            )
            if @xadTestMode
              Builtins.y2milestone("xad:NovellLum:execute command: %1", logCmd2)
            end
            ret = 0
            if @xadTestMode == false
              ret = Convert.to_integer(
                SCR.Execute(
                  path(".target.bash"),
                  Ops.add(upgradeStr, execute_cmd)
                )
              )
            end
            raise Break if ret == 0
            raise Break if @xadTestMode
          end
        end
        Popup.ClearFeedback
        if ret != 0
          # Error message
          errorReport = Builtins.sformat(
            _(
              "Unable to connect to LDAP server %1 or the\n" +
                "specified user %2 does not have enough\n" +
                "privileges to configure Linux User Management.\n" +
                "Error returned is %3:%4.\n" +
                "Please correct the problem and re-run namconfig\n" +
                "after the install. Other products dependent on LUM\n" +
                "will also need to be reconfigured."
            ),
            Ops.get_string(primary_server, "CONFIG_LDAP_SERVER_IP", ""),
            admin_fdn,
            ret,
            NovellUtils.TranslateLDAPError(ret)
          )
          Report.Error(errorReport)
        end
        UpdatePamServiceConf()

        #		if( restrict_access == "yes" )
        #		    ChangeUmask();


        # Modify the /etc/security/pam_unix2.conf file to add call_modules=nam to the auth section
        #Command executed from command line: sed -i 's/\(^[\t ]*auth\:[\t ]*\).*/\1 call_modules=nam/' /etc/security/pam_unix2.conf
        #execute_cmd = "sed -i 's/\\(^[^;#]*[\\t ]*auth\\:[\\t ]*\\).*/\\1 call_modules=nam/' /etc/security/pam_unix2.conf";
        #y2milestone("NovellLum: Executing: %1", execute_cmd);
        #ret = (integer)SCR::Execute(.target.bash, execute_cmd);

        # Remove anything after account:, password: or session:
        #Command executed from command line: sed -i 's/\(^[\t ]*\(account\|password\|session\)\:[\t ]*\).*/\1 /' /etc/security/pam_unix2.conf
        #execute_cmd = "sed -i 's/\\(^[\\t ]*\\(account\\|password\\|session\\)\\:[\\t ]*\\).*/\\1 /' /etc/security/pam_unix2.conf";
        #y2milestone("NovellLum: Executing: %1", execute_cmd);
        #ret = (integer)SCR::Execute(.target.bash, execute_cmd);

        # Remove anything after auth:, password: or session:
        #Command executed from command line: sed -i 's/\(^[\t ]*auth\:[\t ]*\).*/\1 /' /etc/security/pam_unix2.conf
        # Fix for Bug #670564
        #		execute_cmd = "sed -i 's/\\(^[\\t ]*auth\\:[\\t ]*\\).*/\\1 /' /etc/security/pam_unix2.conf";
        #		y2milestone("NovellLum: Executing: %1", execute_cmd);
        #		ret = (integer)SCR::Execute(.target.bash, execute_cmd);
        ################################################################################
        #
        # Modify /etc/nsswitch.conf file to have nam entries for passwd:, shadow:,group
        #
        ################################################################################
        exec_str = "/usr/lib/novell-lum/nsswitch.sh"
        ret = 0
        if @xadTestMode == false
          ret = Convert.to_integer(SCR.Execute(path(".target.bash"), exec_str))
        end
        if ret != 0
          Report.Error(
            "Failed to modify nsswitch.conf.  Add nam keyword on passwd: and group: lines manually."
          )
        end

        if @xadTestMode == false
#          Service.Restart("namcd")
#        ret = Convert.to_integer(SCR.Execute(path(".target.bash"), "rcnamcd restart"))
 
#          Service.RunInitScript("sshd", "restart")

          Service.Restart("sshd")
#        Builtins.y2milestone("namcd and sshd service restarted %1",ret)
          if Stage.cont
            services = []
            Builtins.foreach(["dbus", "haldaemon"]) do |service|
              if Service.Status(service) == 0
                services = Builtins.add(services, service)
              end
            end
            if Ops.greater_than(Builtins.size(services), 0)
              Builtins.y2milestone("services %1 will be restarted", services)
              SCR.Write(
                path(".target.string"),
                Ops.add(Directory.vardir, "/restart_services"),
                Ops.add(Builtins.mergestring(services, "\n"), "\n")
              )
            end
          end
        end

        if !simple
          ################################################################################
          #
          # We need to lum enable the user admin. This has been moved from the NSS code.
          #
          ################################################################################
          # Get just the admin name from the admin_fdn
          admin_name = EdirUtils.GetLeafNameNDS(admin_fdn)
          # Get the context of the admin user from the admin FDN
          admin_ctx = NovellUtils.GetContextNDS(admin_fdn)
          admin_ctx = NovellUtils.Nds2Ldap(admin_ctx)
          Builtins.y2milestone(
            "NovellLum:admin context for lum enablement: %1",
            admin_ctx
          )
          hostname = NovellUtils.GetHostname
          #string groupContext = admin_ctx;
          admin_group = Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
          admingroup_name = EdirUtils.GetLeafNameNDS(admin_group)
          groupContext = ValidationHelpers.toLdap(
            NovellUtils.GetContextNDS(admin_group)
          )

          if @xadMode
            # Recalculating and use the default group context since user did not change it. We recalculate due to a possible eDir failure where server context could have changed since read phase.
            @userChangedAdminGroup = true if Mode.mode == "autoinstallation"
            if @userChangedAdminGroup == false # don't modify it if they typed something in the box -- just use it.
              groupContext = GetAdminGroupContext()
            else
              Builtins.y2milestone(
                "NovellLum.groupContext.Skipping recalculation of group context default since user entered their choice."
              )
            end
          end

          mRet = {}
          execute_cmd = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        "namgroupadd -a '",
                        NovellUtils.Nds2Ldap(admin_fdn)
                      ),
                      "' -x '"
                    ),
                    groupContext
                  ),
                  "' -W "
                ),
                hostname
              ),
              " "
            ),
            admingroup_name
          )
          Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
          execute_cmd = Ops.add(
            Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
            execute_cmd
          )
          Builtins.y2debug("LUM Apply Settings, cmd = %1", execute_cmd)
          ret2 = 0

          Builtins.foreach([1, 2, 3]) do |i|
            if @xadTestMode
              Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
            else
              mRet = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), execute_cmd)
              )
              ret2 = Ops.get_integer(mRet, "exit", -1)
            end
            if Ops.get_string(mRet, "stdout", "") ==
                "Group with the same name exists\n"
              raise Break
            end
            raise Break if ret2 == 0
            raise Break if @xadTestMode
            Builtins.y2milestone("NovellLum::sleeping 5 seconds")
            Builtins.sleep(5000)
          end

          if ret2 != 0
            Builtins.y2milestone(
              "NovellLum:namgroupadd failed: %1. stdout: %2. Trying namgroupmod.",
              ret2,
              Ops.get_string(mRet, "stdout", "")
            )

            tryAgain = true
            while tryAgain == true
              namGroupModWorked = false
              execute_cmd = Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              "namgroupmod -a '",
                              NovellUtils.Nds2Ldap(admin_fdn)
                            ),
                            "' -W "
                          ),
                          hostname
                        ),
                        " 'cn="
                      ),
                      admingroup_name
                    ),
                    ","
                  ),
                  groupContext
                ),
                "'"
              )
              Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
              execute_cmd = Ops.add(
                Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
                execute_cmd
              )
              if @xadTestMode
                Builtins.y2milestone(
                  "xad:NovellLum:execute command: %1",
                  execute_cmd
                )
              end

              Popup.ShowFeedback(
                _("Please Wait"),
                _(
                  "Running namgroupmod for admin group. This may take a while..."
                )
              )

              # retry loop -- 3 attempts with 10 second delays between tries.
              Builtins.foreach([1, 2, 3]) do |i|
                ret2 = 0
                if !@xadTestMode
                  mRet = Convert.to_map(
                    SCR.Execute(path(".target.bash_output"), execute_cmd)
                  )
                  ret2 = Ops.get_integer(mRet, "exit", -1)
                else
                  ret2 = 0
                end
                if ret2 != 0
                  Builtins.y2milestone(
                    "NovellLum:namgroupmod failed (attempt # %1 of 8): %2. stdout: %3",
                    i,
                    ret2,
                    Ops.get_string(mRet, "stdout", "")
                  )
                else
                  Builtins.y2milestone(
                    "NovellLum:namgroupmod succeeded: %1",
                    mRet
                  )
                  namGroupModWorked = true
                  tryAgain = false
                  raise Break
                end
                Builtins.sleep(10000) if Ops.less_or_equal(i, 3) #sleep 10 seconds
              end # end for

              if namGroupModWorked == false
                totalRetry = 3
                # retry loop -- 5 attempts with 30 second delays between tries.
                Builtins.foreach([1, 2, 3, 4, 5]) do |i|
                  totalRetry = Ops.add(totalRetry, 1)
                  ret2 = 0
                  if !@xadTestMode
                    mRet = Convert.to_map(
                      SCR.Execute(path(".target.bash_output"), execute_cmd)
                    )
                    ret2 = Ops.get_integer(mRet, "exit", -1)
                  else
                    ret2 = 0
                  end
                  if ret2 != 0
                    Builtins.y2milestone(
                      "NovellLum:namgroupmod failed (attempt # %1 of 8): %2. stdout: %3",
                      totalRetry,
                      ret2,
                      Ops.get_string(mRet, "stdout", "")
                    )
                  else
                    Builtins.y2milestone(
                      "NovellLum:namgroupmod succeeded: %1",
                      mRet
                    )
                    namGroupModWorked = true
                    tryAgain = false
                    raise Break
                  end
                  Builtins.sleep(30000) if Ops.less_than(i, 5) #sleep 30 seconds
                end # end for
              end

              Popup.ClearFeedback

              if namGroupModWorked == false
                whatnext = Popup.AnyQuestion3(
                  _("Unable to LUM modify 'admingroup'"),
                  _("You can abort, ignore, or retry the configuration of LUM."),
                  _("&Retry"),
                  _("&Ignore"),
                  _("&Abort"),
                  :focus_yes
                )
                if whatnext != :yes # != Retry which means Ignore or Abort
                  Report.Error(
                    _(
                      "Please correct the problem and run the LUM config after the install completes.\nOther products dependent on LUM may need to be reconfigured as well."
                    )
                  )
                  if whatnext != :no # this means they chose to abort the LUM install.
                    Builtins.y2milestone(
                      "NovellLum:user has chosen to Abort the configuration of LUM."
                    )
                    return true
                  else
                    Builtins.y2milestone(
                      "NovellLum:user has chosen to Ignore the namgroupmod error and continue the configuration of LUM."
                    )
                    tryAgain = false # They have chosen to Ignore the error.
                  end
                else
                  Builtins.y2milestone(
                    "NovellLum:user has chosen to try namgroupmod retry loops again."
                  )
                end
              end
            end # end while
          end

          execute_cmd = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              Ops.add(
                                "namuseradd -a '",
                                NovellUtils.Nds2Ldap(admin_fdn)
                              ),
                              "' -x '"
                            ),
                            admin_ctx
                          ),
                          "' -g "
                        ),
                        "'cn="
                      ),
                      admingroup_name
                    ),
                    ","
                  ),
                  groupContext
                ),
                "' '"
              ),
              admin_name
            ),
            "'"
          )
          Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
          execute_cmd = Ops.add(
            Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
            execute_cmd
          )
          if @xadTestMode
            Builtins.y2milestone(
              "xad:NovellLum:execute command: %1",
              execute_cmd
            )
          end
          ret2 = 0
          if @xadTestMode == false
            mRet = Convert.to_map(
              SCR.Execute(path(".target.bash_output"), execute_cmd)
            )
            ret2 = Ops.get_integer(mRet, "exit", -1)
          end

          if ret2 != 0
            Builtins.y2milestone(
              "NovellLum:namuseradd returned an errorcode. This may be normal if the user already exists: %1. stdout: %2",
              ret2,
              Ops.get_string(mRet, "stdout", "")
            )
            execute_cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              "namusermod -a '",
                              NovellUtils.Nds2Ldap(admin_fdn)
                            ),
                            "' -g "
                          ),
                          "'cn="
                        ),
                        admingroup_name
                      ),
                      ","
                    ),
                    groupContext
                  ),
                  "' '"
                ),
                NovellUtils.Nds2Ldap(admin_fdn)
              ),
              "'"
            )
            Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
            execute_cmd = Ops.add(
              Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
              execute_cmd
            )
            ret2 = 0
            if @xadTestMode
              Builtins.y2milestone(
                "xad:NovellLum:execute command: %1",
                execute_cmd
              )
            else
              mRet = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), execute_cmd)
              )
              ret2 = Ops.get_integer(mRet, "exit", -1)
            end

            if ret2 != 0
              Builtins.y2milestone(
                "NovellLum:namusermod failed: %1. stdout: %2",
                ret2,
                Ops.get_string(mRet, "stdout", "")
              )
              Report.Error(
                Builtins.sformat(
                  _(
                    "Unable to LUM add/modify '%1'.\n" +
                      "Please correct the problem and run the LUM config after the install completes.\n" +
                      "Other products dependent on LUM may need to be reconfigured as well."
                  ),
                  admin_name
                )
              )
            end
          end
          ########################################################################################
          #
          #   LUM Enable the 'Domain Users' group and associate with work station context.
          #   if we are installing in xad mode.
          #
          ########################################################################################
          if @xadMode
            # The following call when entered on the command line for the given domain name.
            # namgroupmod -a 'cn=admin,o=novell' -W bgarrett83 'cn=Domain Users,cn=Users,ou=lum,o=novell'

            ipaddress = Ops.get_string(
              primary_server,
              "CONFIG_LDAP_SERVER_IP",
              ""
            )
            ipaddressLocal = NovellUtils.GetIPAddr
            port2 = Builtins.sformat(
              "%1",
              Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)
            )

            admin = Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_ADMIN_CONTEXT",
              ""
            )
            pword = Ops.get_string(
              NovellLDAP.Settings,
              "CONFIG_LDAP_ADMIN_PASSWORD",
              ""
            )

            xadDomainName = ValidationHelpers.toLdap(GetXadDomainName())
            domainType = GetDomainType()
            groupName = "cn=Domain Users"

            grpContext = ""
            if domainType == "controller" || domainType == "domain"
              nameContext = GetNameContextFromServerContext(GetServerContext())
              grpContext = Ops.add(
                Ops.add("'cn=Domain Users,cn=Users,", nameContext),
                "'"
              ) # domainType == "forest"
            else
              nameMapContext = GetNameMapContainer()
              grpContext = Ops.add(
                Ops.add("'cn=Domain Users,cn=Users,", xadDomainName),
                "'"
              )
              if GetNameMap() == "yes"
                grpContext = Ops.add(
                  Ops.add("'cn=Domain Users,cn=Users,", nameMapContext),
                  "'"
                )
              end
            end

            # Modify the LUM enabled "Domain Users" group
            execute_cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add("namgroupmod -a '", NovellUtils.Nds2Ldap(admin_fdn)),
                    "' -W "
                  ),
                  hostname
                ),
                " "
              ),
              grpContext
            )
            Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
            execute_cmd = Ops.add(
              Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
              execute_cmd
            )
            ret2 = 0
            if @xadTestMode
              Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
            else
              mRet = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), execute_cmd)
              )
              ret2 = Ops.get_integer(mRet, "exit", -1)
            end

            if ret2 != 0
              Builtins.y2milestone(
                "NovellLum:namgroupmod failed: %1. stdout: %2",
                ret2,
                Ops.get_string(mRet, "stdout", "")
              )
              Report.Error(
                Builtins.sformat(
                  _(
                    "Unable to LUM add/modify '%1'.\n" +
                      "Please correct the problem and run the LUM config after the install completes.\n" +
                      "Other products dependent on LUM may need to be reconfigured as well."
                  ),
                  grpContext
                )
              )
            end

            # Now make administrator a member of the group we lum enabled
            execute_cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        "namusermod -a '",
                        NovellUtils.Nds2Ldap(admin_fdn)
                      ),
                      "' -g "
                    ),
                    grpContext
                  ),
                  " '"
                ),
                NovellUtils.Nds2Ldap(admin_fdn)
              ),
              "'"
            )
            Builtins.y2milestone("NovellLum:execute command: %1", execute_cmd)
            execute_cmd = Ops.add(
              Ops.add(Ops.add("LUM_PWD='", pwd), "' "),
              execute_cmd
            )
            if @xadTestMode
              Builtins.y2milestone(
                "xad:NovellLum:execute command: %1",
                execute_cmd
              )
            end
            ret2 = 0
            if @xadTestMode == false
              mRet = Convert.to_map(
                SCR.Execute(path(".target.bash_output"), execute_cmd)
              )
              ret2 = Ops.get_integer(mRet, "exit", -1)
            end

            if ret2 != 0
              Builtins.y2milestone(
                "NovellLum:namusermod failed: %1. stdout: %2",
                ret2,
                Ops.get_string(mRet, "stdout", "")
              )
              Report.Error(
                Builtins.sformat(
                  _(
                    "Unable to make administrator '%1' member of group '%2'.\n" +
                      "Please correct the problem and run the LUM config after the install completes.\n" +
                      "Other products dependent on LUM may need to be reconfigured as well."
                  ),
                  admin_fdn,
                  grpContext
                )
              )
            end
          end
        end # LUM upgrade from OES2 sp1
      else
        Builtins.y2milestone("NovellLum:skipped lum config for upgrade.")

        #This code needs to be executed while upgrade to oes2018 only

        if Mode.update || Mode.autoupgrade

          Builtins.y2milestone("Setting commonproxy username and password during upgrade")

          lum_proxy_user = Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
          proxyUser = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          if lum_proxy_user == proxyUser && (lum_proxy_user != "" || proxyUser != "")
            Builtins.y2milestone("NovellLum:Setting commonproxy username and password during upgrade")
            proxyPassword = NovellUtils.RetrieveProxyCredentials(
                "password" )

            ret = NovellUtils.StoreCredsInOCSforUpgrade(proxyUser, proxyPassword,"novell-lum")
            if ret != 0
              Popup.Message(
              Builtins.sformat(
               _("Error storing credentials in OCS for Common Proxy User: %1"),ret ))
              Builtins.y2milestone(
                "Novell-lum.ConfigureProxyUser:Error storing credentials in OCS for common Proxy User: %1",ret)
            end
          elsif lum_proxy_user != "" 
            Builtins.y2milestone("NovellLum:LUM using service proxy, moving to common proxy")
            NovellUtils.MovetoCommonProxyDuringUpgrade("novell-lum")
            Ops.set(@Settings, "CONFIG_LUM_PROXY_USER", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          end
        end
        if Mode.update || Mode.autoupgrade
          lum_proxy_user = Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
          proxyUser = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))

          if lum_proxy_user != "" && lum_proxy_user != proxyUser
            Builtins.y2milestone("NovellLum:LUM using service proxy, moving to common proxy")
            NovellUtils.MovetoCommonProxyDuringUpgrade("novell-lum")
            Ops.set(@Settings, "CONFIG_LUM_PROXY_USER", Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_PROXY_CONTEXT", ""))
          end
        end

        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_LDAP_SERVER"),
          Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_PARTITION_ROOT"),
          Ops.get_string(@Settings, "CONFIG_LUM_PARTITION_ROOT", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_WS_CONTEXT"),
          Ops.get_string(@Settings, "CONFIG_LUM_WS_CONTEXT", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_PROXY_USER"),
          Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_RESTRICT_ACCESS"),
          Ops.get_string(@Settings, "CONFIG_LUM_RESTRICT_ACCESS", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_LOGIN"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_LOGIN", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_FTP"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_FTP", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SSHD"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SSHD", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SU"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SU", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_SFCB"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SFCB", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GDM"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GDM", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_SERVICE_GNOMESU_PAM"),
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GNOMESU_PAM", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_ADMIN_GROUP"),
          Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_ALTERNATE_SERVERS_LIST1"),
          Builtins.mergestring(
            Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST1", []),
            ","
          )
        )
        SCR.Write(
          path(".sysconfig.novell-lum.CONFIG_LUM_ALTERNATE_SERVERS_LIST2"),
          Builtins.mergestring(
            Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST2", []),
            ","
          )
        )
        restrict_access = Ops.get_string(
          @Settings,
          "CONFIG_LUM_RESTRICT_ACCESS",
          ""
        )
        if restrict_access == "yes"
          ret2 = Convert.to_integer(
            SCR.Execute(path(".target.bash"), "namconfig set umask=0077")
          )
          if ret2 == 0
            Builtins.y2milestone("updated /etc/nam.conf to set umask=0077")
          else
            Builtins.y2milestone(
              "failed to update umask value in /etc/nam.conf"
            )
          end
        end

        if @xadTestMode == false
#          Service.Restart("namcd")
#        ret = Convert.to_integer(SCR.Execute(path(".target.bash"), "rcnamcd restart"))

          Service.Restart("sshd")
          if Stage.cont
            services = []
            Builtins.foreach(["dbus", "haldaemon"]) do |service|
              if Service.Status(service) == 0
                services = Builtins.add(services, service)
              end
            end
            if Ops.greater_than(Builtins.size(services), 0)
              Builtins.y2milestone("services %1 will be restarted", services)
              SCR.Write(
                path(".target.string"),
                Ops.add(Directory.vardir, "/restart_services"),
                Ops.add(Builtins.mergestring(services, "\n"), "\n")
              )
            end
          end
        end
      end

      SCR.Write(path(".sysconfig.novell-lum.SERVICE_CONFIGURED"), "yes")
      # Flush file
      SCR.Write(path(".sysconfig.novell-lum"), nil)

      Service.Stop("nscd")
      Service.Start("nscd")
      Builtins.y2milestone("stopped and  Started nscd service")

      Service.Enable("namcd")
      Service.Start("namcd")
      Builtins.y2milestone("Enabled and Started namcd service")

      return false if Abort()
      # Progress finished
      Progress.NextStage
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()
      true
    end


    # Get all novell-lum settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "novell-lum", false)
        return true
      end

      @doRecalculate = false # don't recalculate the ws context in Write() since we are getting the value from the autoYaST control file.

      Ops.set(
        @Settings,
        "CONFIG_LUM_LDAP_SERVER",
        Ops.get_string(settings, "ldap_server", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_WS_CONTEXT",
        Ops.get_string(settings, "ws_context", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_PARTITION_ROOT",
        Ops.get_string(settings, "partition_root", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_RESTRICT_ACCESS",
        Ops.get_string(settings, "restrict_access", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_ADMIN_GROUP",
        Ops.get_string(settings, "admin_group", "")
      )

      Ops.set(
        @Settings,
        "CONFIG_LUM_ALTERNATE_SERVERS_LIST1",
        Ops.get_list(settings, "alternate_ldap_servers_list1", [])
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_ALTERNATE_SERVERS_LIST2",
        Ops.get_list(settings, "alternate_ldap_servers_list2", [])
      )

      lum_enabled_services = Ops.get_map(settings, "lum_enabled_services", {})
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_LOGIN",
        Ops.get_string(lum_enabled_services, "login", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_FTP",
        Ops.get_string(lum_enabled_services, "ftp", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_SSHD",
        Ops.get_string(lum_enabled_services, "sshd", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_SU",
        Ops.get_string(lum_enabled_services, "su", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_SFCB",
        Ops.get_string(lum_enabled_services, "sfcb", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_GDM",
        Ops.get_string(lum_enabled_services, "gdm", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER",
        Ops.get_string(lum_enabled_services, "gnome-screensaver", "no")
      )
      Ops.set(
        @Settings,
        "CONFIG_LUM_SERVICE_GNOMESU_PAM",
        Ops.get_string(lum_enabled_services, "gnomesu-pam", "no")
      )

      @globals_configured = true

      true
    end

    # Dump the novell-lum settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-lum") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-lum", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-lum", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", "")
        )
        Ops.set(
          settings,
          "ws_context",
          Ops.get_string(@Settings, "CONFIG_LUM_WS_CONTEXT", "")
        )
        Ops.set(
          settings,
          "partition_root",
          Ops.get_string(@Settings, "CONFIG_LUM_PARTITION_ROOT", "")
        )
        Ops.set(
          settings,
          "restrict_access",
          Ops.get_string(@Settings, "CONFIG_LUM_RESTRICT_ACCESS", "")
        )
        Ops.set(
          settings,
          "admin_group",
          Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
        )

        Ops.set(
          settings,
          "alternate_ldap_servers_list1",
          Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST1", [])
        )
        Ops.set(
          settings,
          "alternate_ldap_servers_list2",
          Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST2", [])
        )

        lum_enabled_services = {}
        Ops.set(
          lum_enabled_services,
          "login",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_LOGIN", "no")
        )
        Ops.set(
          lum_enabled_services,
          "ftp",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_FTP", "no")
        )
        Ops.set(
          lum_enabled_services,
          "sshd",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SSHD", "no")
        )
        Ops.set(
          lum_enabled_services,
          "su",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SU", "no")
        )
        Ops.set(
          lum_enabled_services,
          "sfcb",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SFCB", "no")
        )
        Ops.set(
          lum_enabled_services,
          "gdm",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GDM", "no")
        )
        Ops.set(
          lum_enabled_services,
          "gnome-screensaver",
          Ops.get_string(
            @Settings,
            "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER",
            "no"
          )
        )
        Ops.set(
          lum_enabled_services,
          "gnomesu-pam",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GNOMESU_PAM", "no")
        )

        Ops.set(settings, "lum_enabled_services", lum_enabled_services)
      end

      deep_copy(settings)
    end

    # Dump all of the novell-lum settings to a single map
    # (For use by proposal.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def ExportSettings
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-lum") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "novell-lum", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "novell-lum", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", "")
        )
        Ops.set(
          settings,
          "ws_context",
          Ops.get_string(@Settings, "CONFIG_LUM_WS_CONTEXT", "")
        )
        Ops.set(
          settings,
          "partition_root",
          Ops.get_string(@Settings, "CONFIG_LUM_PARTITION_ROOT", "")
        )
        Ops.set(
          settings,
          "restrict_access",
          Ops.get_string(@Settings, "CONFIG_LUM_RESTRICT_ACCESS", "")
        )
        Ops.set(
          settings,
          "admin_group",
          Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
        )
        Ops.set(
          settings,
          "alternate_ldap_servers_list1",
          Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST1", [])
        )
        Ops.set(
          settings,
          "alternate_ldap_servers_list2",
          Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST2", [])
        )

        lum_enabled_services = {}
        Ops.set(
          lum_enabled_services,
          "login",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_LOGIN", "no")
        )
        Ops.set(
          lum_enabled_services,
          "ftp",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_FTP", "no")
        )
        Ops.set(
          lum_enabled_services,
          "sshd",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SSHD", "no")
        )
        Ops.set(
          lum_enabled_services,
          "su",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SU", "no")
        )
        Ops.set(
          lum_enabled_services,
          "sfcb",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SFCB", "no")
        )
        Ops.set(
          lum_enabled_services,
          "gdm",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GDM", "no")
        )
        Ops.set(
          lum_enabled_services,
          "gnomesu-pam",
          Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GNOMESU_PAM", "no")
        )

        Ops.set(settings, "lum_enabled_services", lum_enabled_services)
      end

      deep_copy(settings)
    end

    # Tranlate a yes or no string
    # @return translated "yes" or "no" string
    def Trans_yes_no(yesno)
      if Builtins.tolower(yesno) == "yes"
        return _("yes")
      else
        return _("no")
      end
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        admin_fdn = NovellUtils.Nds2Ldap(
          Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_ADMIN_CONTEXT", "")
        )
        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "NovellLum:XAD_TREE_ADMIN_CONTEXT(summary):%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            admin_fdn = NovellUtils.Nds2Ldap(xad_tree_admin_context)
          end
        end
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(_("LDAP admin name with context: %1"), admin_fdn)
        )

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("LDAP server IP address / hostname: %1"),
            Ops.get_string(@Settings, "CONFIG_LUM_LDAP_SERVER", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Linux/Unix config context: %1"),
            Ops.get_string(@Settings, "CONFIG_LUM_PARTITION_ROOT", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("LUM workstation context: %1"),
            Ops.get_string(@Settings, "CONFIG_LUM_WS_CONTEXT", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Admin group name with context: %1"),
            Ops.get_string(@Settings, "CONFIG_LUM_ADMIN_GROUP", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Proxy user name with context: %1"),
            Ops.get_string(@Settings, "CONFIG_LUM_PROXY_USER", "")
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Restrict read and write access of LUM enabled users: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_RESTRICT_ACCESS", "")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          _("PAM-enabled services to allow authentication via eDirectory:")
        )

        # To display alternate_server_list
        alternate_server_list = ""
        if Ops.greater_than(
            Builtins.size(
              Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST1", [])
            ),
            0
          )
          Builtins.foreach(
            Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST1", [])
          ) do |ip|
            alternate_server_list = Ops.add(
              Ops.add(ip, ","),
              alternate_server_list
            )
          end
        end
        if Ops.greater_than(
            Builtins.size(
              Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST2", [])
            ),
            0
          )
          Builtins.foreach(
            Ops.get_list(@Settings, "CONFIG_LUM_ALTERNATE_SERVERS_LIST2", [])
          ) do |ip|
            alternate_server_list = Ops.add(
              Ops.add(ip, ","),
              alternate_server_list
            )
          end
        end
        alternate_server_list = Builtins.substring(
          alternate_server_list,
          0,
          Ops.subtract(Builtins.size(alternate_server_list), 1)
        )

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("Alternate Ldap server: %1"),
            alternate_server_list
          )
        )

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("login: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_LOGIN", "no")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("ftp: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_FTP", "no")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("sshd: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SSHD", "no")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("su: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SU", "no")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("sfcb: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_SFCB", "yes")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("gdm: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GDM", "no")
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("gnome-screensaver: %1"),
            Trans_yes_no(
              Ops.get_string(
                @Settings,
                "CONFIG_LUM_SERVICE_GNOME_SCREENSAVER",
                "no"
              )
            )
          )
        )
        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("gnomesu-pam: %1"),
            Trans_yes_no(
              Ops.get_string(@Settings, "CONFIG_LUM_SERVICE_GNOMESU_PAM", "no")
            )
          )
        )

        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :variable => :old_EdirSettings, :type => "map"
    publish :variable => :old_XadSettings, :type => "map"
    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :edir_service_configured, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :schemaLogPath, :type => "string"
    publish :variable => :oesContext, :type => "string"
    publish :variable => :lum_proxy, :type => "boolean"
    publish :variable => :userChangedData, :type => "boolean"
    publish :variable => :userChangedAdminGroup, :type => "boolean"
    publish :variable => :doRecalculate, :type => "boolean"
    publish :function => :Modified, :type => "boolean ()"
    publish :function => :PollAbort, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :GetAdminGroupContext, :type => "string ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :GetNameContextFromServerContext, :type => "string (string)"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :ExportSettings, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  NovellLum = NovellLumClass.new
  NovellLum.main
end
